import { IGNORED_RUNTIME_DEPENDENCIES } from '../constants.js';
import { debugLog } from './debug.js';
import { toDebugString } from './input.js';
import { fromBinary, isBinary, isConfigPattern, isDeferResolveEntry, isDependency } from './input.js';
import { getPackageNameFromSpecifier } from './modules.js';
import { dirname, isAbsolute, isInternal, join } from './path.js';
import { _resolveSync } from './resolve.js';
export const getReferencedDependencyHandler = (collector, deputy, chief, isGitIgnored) => (input, workspace) => {
    const { specifier, containingFilePath } = input;
    if (!containingFilePath || IGNORED_RUNTIME_DEPENDENCIES.has(specifier))
        return;
    if (isBinary(input)) {
        const binaryName = fromBinary(input);
        const ws = (input.dir && chief.findWorkspaceByFilePath(`${input.dir}/`)) || workspace;
        const isHandled = deputy.maybeAddReferencedBinary(ws, binaryName);
        if (isHandled)
            return;
        collector.addIssue({
            type: 'binaries',
            filePath: containingFilePath,
            workspace: workspace.name,
            symbol: binaryName,
        });
        return;
    }
    const packageName = getPackageNameFromSpecifier(specifier);
    if (packageName) {
        const isWorkspace = chief.workspacesByPkgName.has(packageName);
        const specifierWorkspace = chief.findWorkspaceByFilePath(containingFilePath) ?? workspace;
        if (specifierWorkspace) {
            const isHandled = deputy.maybeAddReferencedExternalDependency(specifierWorkspace, packageName);
            if (isWorkspace || isDependency(input)) {
                if (!isHandled) {
                    if ((deputy.isProduction && input.production) || !deputy.isProduction) {
                        collector.addIssue({
                            type: 'unlisted',
                            filePath: containingFilePath,
                            workspace: specifierWorkspace.name,
                            symbol: specifier,
                        });
                    }
                    return;
                }
                const ref = _resolveSync(specifier, dirname(containingFilePath));
                if (ref && !isGitIgnored(ref))
                    return ref;
            }
            if (isHandled)
                return;
        }
    }
    if (!isConfigPattern(input) && deputy.isProduction && !input.production) {
        return;
    }
    const baseDir = input.dir ?? dirname(containingFilePath);
    const filePath = isAbsolute(specifier) ? specifier : join(baseDir, specifier);
    const resolvedFilePath = _resolveSync(filePath, baseDir);
    if (resolvedFilePath && isInternal(resolvedFilePath)) {
        return isGitIgnored(resolvedFilePath) ? undefined : resolvedFilePath;
    }
    if (!isInternal(filePath)) {
        collector.addIssue({
            type: 'unlisted',
            filePath: containingFilePath,
            workspace: workspace.name,
            symbol: packageName ?? specifier,
        });
    }
    else if (!isGitIgnored(filePath)) {
        if (!isDeferResolveEntry(input) && !isConfigPattern(input)) {
            collector.addIssue({
                type: 'unresolved',
                filePath: containingFilePath,
                workspace: workspace.name,
                symbol: specifier,
            });
        }
        else {
            debugLog(workspace.name, `Unable to resolve ${toDebugString(input)}`);
        }
    }
};
